//-------------------------------------------------------------------//
//                                                                   //
//  JFC Copyright (C) Glenn Rosenthal, 1999,2000,2001.               //
//  All rights reserved.                                             //
//                                                                   //
//-------------------------------------------------------------------//

//-------------------------------------------------------------------//
//
//  This module contains the configuration manipulation routines.
//

#include "jfc_main.h"
#include "jfc_conf.h"
#include "jfc_edit.h"
#include "jfc_file.h"
#include "jfc_help.h"
#include "jfc_misc.h"
#include "jfc_stat.h"
#include <commctrl.h>   // Needed for tab controls
#include <limits.h>     // Used for size of intagers

//-------------------------------------------------------------------
//
//  Static data.
//

#define NAME_CONFIG     TEXT("JFC.cfg")

//
//  This is the default configuation stored as a static structure.
//  This is harder to maintain than a set of assignments, but it is 
//  smaller, and gives us the correct arguments to default intialization,
//  and to reset to default configuration as the user is ruingin.  
//  
//  See the section on the options for a description of processing of 
//  this structure.
//
#ifdef WINCE_PPC
  #define SHORT_NAME    true
#else  WINCE_PPC
  #define SHORT_NAME    false
#endif WINCE_PPC

static struct cfg default_config = {
  CONFIG_MAGIC,                         // long  magic;                  // Identifies this as a JWPce config file.
  true,                                 // byte  install;                // If set causes check for installed version and file extensions.
  true,                                 // byte  status;                 // Generate a status bar
  SHORT_NAME,                           // byte  short_name;             // Use short names in the title bar.
  false,                                // byte  maximize;               // Maximaize the file.
  true,                                 // byte  usedims;                // Use last saved dimensions.
  true,                                 // byte  save_exit;              // Save configuration on exit.     
  true,                                 // byte  reload_files;           // Reload files loaded when we exited.
  EXIT_ASK,                             // byte  exit_open_file;         // Exit quietly even if user is in a card set.
  true,                                 // byte  flash_kanji;            // Show kanji and let user guess kana & meaning
  true,                                 // byte  flash_kana;             // Show kana and let user guess kanji & meaning
  true,                                 // byte  flash_ascii;            // Show ascii and let user guess kana & kanji
  true,                                 // byte  kanji_kanji;            // Show kanji and let user guess readings & meaning (kanji cards)
  true,                                 // byte  kanji_kana;             // Show readings and let user guess kanji & meaning (kanji cards)
  true,                                 // byte  kanji_ascii;            // Show ascii and let user guess readings & kanji (kanji cards)
  false,                                // byte  kanji_always;           // Always show kanji with kana.
  true,                                 // byte  kanji_unknown;          // Only show kanji witht he kana when it is unknown
  false,                                // byte  know_all_kanji;         // Know all kanji
  true,                                 // byte  full_kanjicard;         // Show advanced kanji cards as full headers
  false,                                // byte  duplicate_cards;        // Duplicate cards that are missed.
  DOUBLE_INFO,                          // byte  double_click;           // Determines action when a double click is detected.
  MODE_SWAP,                            // byte  multi_mode;             // Display mode used for multi mode.
  true,                                 // byte  knownkanji_colorkanji;  // Assume that color kanji indicates the known kanji.
  COLORKANJI_NOMATCH,                   // byte  colorkanji_mode;        // Determines the way color-fonts are suported.
  false,                                // byte  swap_mouse;             // Swap elements on mouse down/up
  true,                                 // byte  review_mode;            // Use review mode and the weights.
  true,                                 // byte  keep_stats;             // Keep statistics.
  true,                                 // byte  stat_config;            // Use configuation from status file.
  false,                                // byte  no_move_end;            // Do not place wrong card at the end.
  false,                                // byte  time_cards;             // If user does not answer in time the card is wrong.
  false,                                // byte  use_max;                // Limit number of cards generated.
  false,                                // byte  all_fonts;              // Show all fonts.
  false,                                // byte  cache_displayfont;      // Should we cache or not cache the display font.
  false,                                // byte  cache_systemfont;       // Should we cache or not cache the system font.
  false,                                // byte  kana_for_advanced;      // Uses kana font for advanced kanji compounds.
  true,                                 // byte  info_titles;            // Puts titles in the kanji-info list box.
  false,                                // byte  info_compress;          // Compress Character information.
  false,                                // byte  cache_info;             // Fill for later exapnsion
  false,                                // byte  wheel_control;          // Uses the wheel control to control the program (PPC only)
  false,                                // byte  one_click;              // Use one click simplified controls
  false,                                // byte  auto_expand;            // Automatically expand shown items when moving to the next item.
  false,                                // byte  auto_collapse;          // Automatically collapse shown items when moving to next item
  RGB(0,0,255),                         // COLORREF colorkanji_color;    // Color to be used with color-kanji.
  RGB(255,0,0),                         // COLORREF info_color;          // Color used for titles in kanji-info box.
  CW_USEDEFAULT,                        // int   x,y,xs,ys;              // Dimensions of last saved configuration.
  CW_USEDEFAULT,
  CW_USEDEFAULT,
  CW_USEDEFAULT,                  

  { TEXT("Arial")     ,0 ,true,false }, // TCHAR font[SIZE_NAME];        // Name of roman font.
  { TEXT("k24x24.f00"),24,true,false }, // struct cfg_font kanji_font;   // Kanji font.
  { TEXT("k24x24.f00"),24,true,false }, // struct cfg_font kana_font;    // Kana font
  { TEXT("k24x24.f00"),24,true,false }, // struct cfg_font imi_font;     // meaning font.
  { TEXT("k16x16.f00"),16,true,false }, // struct cfg_font sys_font;     // System font.
  { TEXT("k48x48.f00"),48,true,false }, // struct cfg_font big_font;     // Big font.
  { TEXT("k24x24.f00"),24,true,false }, // struct cfg_font extra_font;   // Extra font for later.
  300,                                  // short display_cache;          // Size of font cache in characters.
  20,                                   // short duplicate_count;        // Frequency for duplicated cards.
  40,                                   // int   maximum;                // Maximum number of cards generated.
  3000,                                 // int   time_show;              // Time to show the card in msec.
  1500,                                 // int   time_next;              // Time to move to the next card in msec.
  1000,                                 // int   max_subfiles;           // Maximum number of sub-files.
  { 25,50,75,100,100,100,100,100,100 }, // byte  flash_kanji[8];         // Flash kanji weights.
  { 25,50,75,100,100,100,100,100,100 }, // byte  flash_kana[8];          // Flash kana weights.
  { 25,50,75,100,100,100,100,100,100 }, // byte  flash_ascii[8];         // Flash ascii weights.
  { 25,50,75,100,100,100,100,100,100 }, // byte  kanji_kanji[8];         // Kanji card kanji weight
  { 25,50,75,100,100,100,100,100,100 }, // byte  kanji_kana[8];          // Kanji card kana weight
  { 25,50,75,100,100,100,100,100,100 }, // byte  kanji_ascii[8];         // Kanji card ascii weight
  { 0 }                                                                  // Fill for later expansion.
};

//-------------------------------------------------------------------
//
//  Static procedures.
//

static TCHAR *get_parameter (TCHAR *buffer,TCHAR *ptr); // Utility rotuine to extract a parameter from a line.

//
//  This routine extracts a parameter from a text line.  A parameter is 
//  defined as a quoted string, or a space deliminated argument.
//
//      buffer -- Buffer to copy parameter to (must be big enough!).
//      ptr    -- Pointer into string being processed.
//
//      RETURN -- Pointer into string being processed at the next 
//                parameter, or pointer to the end of the string.  
//                Generally, this becomes ptr, next time around.
//
static TCHAR *get_parameter (TCHAR *buffer,TCHAR *ptr) {
  TCHAR *p2;
  if (*ptr == '"') {
    lstrcpy (buffer,++ptr);
    for (p2 = buffer; *p2 && (*p2 != '"'); p2++);
  }
  else {
    lstrcpy (buffer,ptr);
    for (p2 = buffer; *p2 && !isspace(*p2); p2++);
  }
  if (!*p2) return (NULL);
  *p2  = 0;
  ptr += (p2-buffer);
  for (ptr++; *ptr && isspace(*ptr); ptr++);
  if (!*ptr) return (NULL);
  return (ptr);
}

//-------------------------------------------------------------------
//
//  This is a small window class for rendering the color samples used
//  in the configuration dialog box.
//

static LRESULT CALLBACK JFC_color_proc (HWND hwnd,UINT iMsg,WPARAM wParam,LPARAM lParam) {
  HDC         hdc;
  PAINTSTRUCT ps;
  RECT        rect;
  HBRUSH      brush;

  switch (iMsg) {
//
//  On mouse down, we send our color to the parent.
//
    case WM_LBUTTONDOWN:
         SendMessage (GetParent(hwnd),WM_COMMAND,GetWindowLong(hwnd,GWL_ID),GetWindowLong(hwnd,0));
         return (0);
//
//  This is the set command that sets the color we are to display.
//
    case WMU_SETWINDOWVALUE:
         SetWindowLong  (hwnd,0,lParam);
         InvalidateRect (hwnd,NULL,false);
         return (0);
//
//  This does the actual redraw.
//
    case WM_PAINT:
         hdc = BeginPaint (hwnd,&ps);
         brush = CreateSolidBrush(GetWindowLong(hwnd,0));
         GetClientRect (hwnd,&rect);
         FillRect      (hdc,&rect,brush);
         DeleteObject  (brush);
         EndPaint      (hwnd,&ps);
         return (0);
  }
  return (DefWindowProc(hwnd,iMsg,wParam,lParam));
}

//-------------------------------------------------------------------
//
//  Static data for Options dialog box.
//

static struct cfg *config;      // Working configu structure used with setup.
static HFONT       font;        // Indicates new font chosen.

//-------------------------------------------------------------------
//
//  Static routines for the options dialog box.
//
//  Color selector routines.
//

#ifdef WINCE
  #define USE_MYCHOOSECOLOR
#endif WINCE

//
//  This is the dialog box handler for my ChooseColor dialog box.  
//  This dialog box was gnerated specifically for use on Windows CE
//  machines where the system one does not exist.
//
#ifdef USE_MYCHOOSECOLOR

static BOOL CALLBACK dialog_choosecolor (HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam) {
  static COLORREF colors[] = { RGB(255,000,000),RGB(000,255,000),RGB(000,000,255),  // Color array for non-grey colors.
                               RGB(255,255,000),RGB(255,000,255),RGB(000,255,255),
                               RGB(180,000,000),RGB(000,180,000),RGB(000,000,180),
                               RGB(180,180,000),RGB(180,000,180),RGB(000,180,180),
                             };
  int i,j;
  switch (msg) {
//
//  Generate the dialog box.
//
    case WM_INITDIALOG:
         SetWindowLong (hwnd,DWL_USER,lParam);                      // Save input parameter for abort.
         for (i = IDC_CCBOX1 ; i <= IDC_CCBOX12; i++) SendDlgItemMessage (hwnd,i,WMU_SETWINDOWVALUE,0,colors[i-IDC_CCBOX1]);
         for (i = IDC_CCBOX13; i <= IDC_CCBOX24; i++) { 
           j = (255*(i-IDC_CCBOX13))/(IDC_CCBOX24-IDC_CCBOX13);     // Setup grey-scale colors.
           SendDlgItemMessage (hwnd,i,WMU_SETWINDOWVALUE,0,RGB(j,j,j));
         }
goto setup_color;
         return (true);
//
//  Help 
//
    case WM_HELP:
         do_help (hwnd,IDH_INTERFACE_COLOR);
         return  (true);
//
//  Main controls
//
    case WM_COMMAND:
         switch (LOWORD(wParam)) {
           case IDC_CCRED:                  // The edit boxes, simply change the color.
           case IDC_CCGREEN:
           case IDC_CCBLUE: {
                  int r,g,b;
                  input_check (wParam);
                  r = GetDlgItemInt(hwnd,IDC_CCRED  ,&i,false);
                  g = GetDlgItemInt(hwnd,IDC_CCGREEN,&i,false);
                  b = GetDlgItemInt(hwnd,IDC_CCBLUE ,&i,false);
                  SendDlgItemMessage (hwnd,IDC_CCSAMPLE,WMU_SETWINDOWVALUE,0,RGB(r,g,b));
                }
                return (true);
           case IDCANCEL:                   // Cancle returns the default color, passed in as an argument.
                EndDialog (hwnd,GetWindowLong(hwnd,DWL_USER));
                return (true);
           case IDOK:                       // OK returns the current color in the sample box.
                EndDialog (hwnd,GetWindowLong(GetDlgItem(hwnd,IDC_CCSAMPLE),0));
                return (true);
          default:                          // Respond to color change.
                if ((LOWORD(wParam) >= IDC_CCBOX1) && (LOWORD(wParam) <= IDC_CCBOX24)) {
setup_color:;
                  SetDlgItemInt (hwnd,IDC_CCRED  ,GetRValue(lParam),false);
                  SetDlgItemInt (hwnd,IDC_CCGREEN,GetGValue(lParam),false);
                  SetDlgItemInt (hwnd,IDC_CCBLUE ,GetBValue(lParam),false);
                  return        (true);
                }
                break;
         }
         break;
  }
  return (false);
}

#endif USE_MYCHOOSECOLOR

//
//  Internal routine used to get the color of something.
//
//      hwnd   -- Parent dialog window.
//      color  -- Default (current) color selection.
//
//      RETURN -- Return value is the newly choses color, or the 
//                old color if the user aborted.
//
static COLORREF get_color (HWND hwnd,COLORREF color) {
// ### If we make custom colors static and intitialize it, we can have 
// ###   custom colors carried over multible invocations.
#ifdef USE_MYCHOOSECOLOR
  return (DialogBoxParam(language,MAKEINTRESOURCE(IDD_CHOOSECOLOR),hwnd,(DLGPROC) dialog_choosecolor,(LPARAM) color));
#else
  COLORREF    custom_colors[16];
  CHOOSECOLOR choose;
  memset (&choose,0,sizeof(choose));
  choose.lStructSize  = sizeof(choose);
  choose.hwndOwner    = hwnd;
  choose.hInstance    = NULL;					// Was, but V++ 6.0 has problems with this instance;
  choose.rgbResult    = color;
  choose.lpCustColors = custom_colors;
  choose.Flags        = CC_ANYCOLOR | CC_RGBINIT | CC_SOLIDCOLOR;
  InvalidateRect (hwnd,NULL,false);
  if (ChooseColor (&choose)) return (choose.rgbResult); else return (color);
  return (color);
#endif USE_MYCHOOSECOLOR
}

//-------------------------------------------------------------------
//
//  Page routines for the options dialog box.
//

//...................................................................
//
//  General page
//

//
//  General page handler.  This handler processes the general page
//
static BOOL CALLBACK options_general (HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam) {
  static short exit_opts  [] = { IDC_OGASK ,IDC_OGQUIET,IDC_OGSAVE    };
  switch (msg) {
    case WM_INITDIALOG:
         CheckDlgButton (hwnd,IDC_OGRESTOREPOS  ,config->usedims      );
         CheckDlgButton (hwnd,IDC_OGRELOADFILES ,config->reload_files );
         CheckDlgButton (hwnd,IDC_OGSAVESETTINGS,config->save_exit    );
         CheckDlgButton (hwnd,IDC_OGSTATUS      ,config->status       );
         CheckDlgButton (hwnd,IDC_OGSHORTNAME   ,config->short_name   );
         CheckDlgButton (hwnd,exit_opts[config->exit_open_file],true);
         return (true);
    case WM_GETDLGVALUES:
         config->usedims         = IsDlgButtonChecked(hwnd,IDC_OGRESTOREPOS  );
         config->reload_files    = IsDlgButtonChecked(hwnd,IDC_OGRELOADFILES );
         config->save_exit       = IsDlgButtonChecked(hwnd,IDC_OGSAVESETTINGS);
         config->status          = IsDlgButtonChecked(hwnd,IDC_OGSTATUS      );
         config->short_name      = IsDlgButtonChecked(hwnd,IDC_OGSHORTNAME   );
         if      (IsDlgButtonChecked(hwnd,IDC_OGASK  )) config->exit_open_file = EXIT_ASK;
         else if (IsDlgButtonChecked(hwnd,IDC_OGSAVE )) config->exit_open_file = EXIT_SAVE;
         else                                           config->exit_open_file = EXIT_QUIT;
         return (true);
  }
  return (false);
}

//...................................................................
//
//  Controls page
//
static BOOL CALLBACK options_controls (HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam) {
  static short double_opts[] = { IDC_OCINFO,IDC_OCRIGHT,IDC_OCDISABLE };
  switch (msg) {
    case WM_INITDIALOG:
         CheckDlgButton (hwnd,IDC_OCWHEELMODE   ,config->wheel_control);
         CheckDlgButton (hwnd,IDC_OCONECLICK    ,config->one_click    );
         CheckDlgButton (hwnd,double_opts[config->double_click],true);
//
//  Display options
//
         CheckDlgButton (hwnd,IDC_OCMOUSE ,config->swap_mouse);
         if (config->multi_mode == MODE_SWAP) {
           CheckDlgButton (hwnd,IDC_OCSWAP ,true);
           CheckDlgButton (hwnd,IDC_OCCYCLE,false);
           EnableWindow   (GetDlgItem(hwnd,IDC_OCMOUSE),true);
         }
         else {
           CheckDlgButton (hwnd,IDC_OCSWAP ,false);
           CheckDlgButton (hwnd,IDC_OCCYCLE,true);
           EnableWindow   (GetDlgItem(hwnd,IDC_OCMOUSE),false);
         }
         return (true);
    case WM_COMMAND:
         switch (LOWORD(wParam)) {
           case IDC_OCCYCLE:
                EnableWindow (GetDlgItem(hwnd,IDC_OCMOUSE),false);
                return       (true);
           case IDC_OCSWAP:
                EnableWindow (GetDlgItem(hwnd,IDC_OCMOUSE),true);
                return       (true);
         }
         break;
    case WM_GETDLGVALUES:
         config->wheel_control   = IsDlgButtonChecked(hwnd,IDC_OCWHEELMODE   );
         config->one_click       = IsDlgButtonChecked(hwnd,IDC_OCONECLICK    );
         config->swap_mouse      = IsDlgButtonChecked(hwnd,IDC_OCMOUSE);
         config->multi_mode      = IsDlgButtonChecked(hwnd,IDC_OCSWAP) ? MODE_SWAP : MODE_CYCLE;
         if      (IsDlgButtonChecked(hwnd,IDC_OCINFO )) config->double_click   = DOUBLE_INFO;
         else if (IsDlgButtonChecked(hwnd,IDC_OCRIGHT)) config->double_click   = DOUBLE_RIGHT;
         else                                           config->double_click   = DOUBLE_DISABLE;
         return (true);
  }
  return (false);
}

//...................................................................
//
//  Cards page
//

//
//  Cards page handler.  This handler processes the general page
//
static BOOL CALLBACK options_cards (HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam) {
  switch (msg) {
    case WM_INITDIALOG:
         CheckDlgButton (hwnd,IDC_OCKEEPSTATS   ,config->keep_stats);
         CheckDlgButton (hwnd,IDC_OCUSECONFIG   ,config->stat_config);
         CheckDlgButton (hwnd,IDC_OCREVIEW      ,config->review_mode);
         CheckDlgButton (hwnd,IDC_OCUSEMAX      ,config->use_max);
         CheckDlgButton (hwnd,IDC_OCTOEND      ,!config->no_move_end);
         CheckDlgButton (hwnd,IDC_OCDUPLICATE   ,config->duplicate_cards);
         CheckDlgButton (hwnd,IDC_OCUSETIME     ,config->time_cards);
         SetDlgItemInt  (hwnd,IDC_OCMAX         ,config->maximum,false);
         SetDlgItemInt  (hwnd,IDC_OCDUPCOUNT    ,config->duplicate_count,false);
         SetDlgItemInt  (hwnd,IDC_OCTIMESHOW    ,config->time_show/100,false);
         SetDlgItemInt  (hwnd,IDC_OCTIMENEXT    ,config->time_next/100,false);
         EnableWindow (GetDlgItem(hwnd,IDC_OCMAX     ),config->use_max);
         EnableWindow (GetDlgItem(hwnd,IDC_OCDUPCOUNT),config->duplicate_cards);
         EnableWindow (GetDlgItem(hwnd,IDC_OCTIMESHOW),config->time_cards);
         EnableWindow (GetDlgItem(hwnd,IDC_OCTIMENEXT),config->time_cards);
         return (true);
    case WM_COMMAND:
         switch (LOWORD(wParam)) {
           INPUT_CHECK (IDC_OCDUPCOUNT);
           INPUT_CHECK (IDC_OCTIMESHOW);
           INPUT_CHECK (IDC_OCTIMENEXT);
           INPUT_CHECK (IDC_OCMAX);
           case IDC_OCDUPLICATE:
                EnableWindow (GetDlgItem(hwnd,IDC_OCDUPCOUNT),IsDlgButtonChecked(hwnd,IDC_OCDUPLICATE));
                return (true);
           case IDC_OCUSEMAX:
                EnableWindow (GetDlgItem(hwnd,IDC_OCMAX),IsDlgButtonChecked(hwnd,IDC_OCUSEMAX));
                return (true);
           case IDC_OCUSETIME:
                EnableWindow (GetDlgItem(hwnd,IDC_OCTIMESHOW),IsDlgButtonChecked(hwnd,IDC_OCUSETIME));
                EnableWindow (GetDlgItem(hwnd,IDC_OCTIMENEXT),IsDlgButtonChecked(hwnd,IDC_OCUSETIME));
                return (true);
         }
         break;
    case WM_GETDLGVALUES:
         config->keep_stats      = IsDlgButtonChecked(hwnd,IDC_OCKEEPSTATS);
         config->stat_config     = IsDlgButtonChecked(hwnd,IDC_OCUSECONFIG);
         config->review_mode     = IsDlgButtonChecked(hwnd,IDC_OCREVIEW);
         config->use_max         = IsDlgButtonChecked(hwnd,IDC_OCUSEMAX);
         config->time_cards      = IsDlgButtonChecked(hwnd,IDC_OCUSETIME);
         config->no_move_end     =!IsDlgButtonChecked(hwnd,IDC_OCTOEND);
         config->duplicate_cards = IsDlgButtonChecked(hwnd,IDC_OCDUPLICATE);
         config->maximum         = get_int(hwnd,IDC_OCMAX     ,5,INT_MAX,config->maximum);
         config->duplicate_count = get_int(hwnd,IDC_OCDUPCOUNT,5,7000,config->duplicate_count);
         config->time_show       = 100*get_int(hwnd,IDC_OCTIMESHOW,0,INT_MAX/100,config->time_show/100);
         config->time_next       = 100*get_int(hwnd,IDC_OCTIMENEXT,0,INT_MAX/100,config->time_next/100);
         return (true);
  }
  return (false);
}

//...................................................................
//
//  Fonts/Display page
//
                                // Indexes for font types
#define FONT_NONE       -1      // No font being worked with (used for initialization)
#define FONT_SYSTEM     0       // System font
#define FONT_KANA       1       // Kana display font
#define FONT_KANJI      2       // Kanji display font
#define FONT_IMI        3       // Meaning display font
#define FONT_BIGFONT    4       // Big font

static short current_font;      // Indicates the current font the user is working on.

static void check_height  (HWND hwnd);
static void select_string (HWND hwnd,int id,TCHAR *string);

//
//  This routine handles a change in the font the user is setting.  This is also
//  used to set the data and to read the data for the current font.  
//
//  First this routine reads the data for the out-going font type.  Then sets the
//  data for the next font type.  
//
//  If you call this routine without changing the font type, you will just read the 
//  values and then put them back into place.
//
//  This routine is used to initialize.  The current_font (out-going font) is set 
//  to FONT_NONE so no data is read, but the new font is initilize.  Simiuarly, the 
//  routien is used to read the data before the dialog is closed.
//  
static void change_font (HWND hwnd) {
  int   i;
  struct cfg_font *font;
//
//  User is changing the font so lets save the old font information.
//
//  We read all information, including the invalid information.  This is safe on the 
//  read, because we just ignore the fields we don't want.
//
  if (current_font != FONT_NONE) {
    switch (current_font) {
      case FONT_SYSTEM : font = &config->sys_font;   break;
      case FONT_KANA   : font = &config->kana_font;  break;
      case FONT_KANJI  : font = &config->kanji_font; break;
      case FONT_IMI    : font = &config->imi_font;   break;
      case FONT_BIGFONT: font = &config->big_font;   break;
    }
    font->size      = get_int(hwnd,IDC_OFHEIGHT,4,1000,font->size);
    font->automatic = IsDlgButtonChecked(hwnd,IDC_OFAUTO);
    i               = SendDlgItemMessage(hwnd,IDC_OFFONTNAME,CB_GETCURSEL,0,0);
    if (CB_ERR != i) SendDlgItemMessage (hwnd,IDC_OFFONTNAME,CB_GETLBTEXT,i,(LPARAM) font->name);
  }
//  
//  Now setup the new font information.
//
//  Here we need to do each case seperately so we can blank out the fields we dont 
//  want displayed.
//
  switch (current_font = (short) SendDlgItemMessage(hwnd,IDC_OFFONT,CB_GETCURSEL,0,0)) {
    case FONT_SYSTEM:
         EnableWindow   (GetDlgItem(hwnd,IDC_OFAUTO),false);
         CheckDlgButton (hwnd,IDC_OFAUTO    ,false);
         select_string  (hwnd,IDC_OFFONTNAME,config->sys_font.name);
         SetDlgItemInt  (hwnd,IDC_OFHEIGHT  ,config->sys_font.size,false);
         break;
    case FONT_KANA:
         EnableWindow   (GetDlgItem(hwnd,IDC_OFAUTO),true);
         CheckDlgButton (hwnd,IDC_OFAUTO    ,config->kana_font.automatic );
         select_string  (hwnd,IDC_OFFONTNAME,config->kana_font.name      );
         SetDlgItemInt  (hwnd,IDC_OFHEIGHT  ,config->kana_font.size,false);
         break;
    case FONT_KANJI:
         EnableWindow   (GetDlgItem(hwnd,IDC_OFAUTO),true);
         CheckDlgButton (hwnd,IDC_OFAUTO    ,config->kanji_font.automatic );
         select_string  (hwnd,IDC_OFFONTNAME,config->kanji_font.name      );
         SetDlgItemInt  (hwnd,IDC_OFHEIGHT  ,config->kanji_font.size,false);
         break;
    case FONT_IMI:
         EnableWindow   (GetDlgItem(hwnd,IDC_OFAUTO),true);
         CheckDlgButton (hwnd,IDC_OFAUTO    ,config->imi_font.automatic );
         select_string  (hwnd,IDC_OFFONTNAME,config->imi_font.name      );
         SetDlgItemInt  (hwnd,IDC_OFHEIGHT  ,config->imi_font.size,false);
         break;
    case FONT_BIGFONT:
         EnableWindow   (GetDlgItem(hwnd,IDC_OFAUTO),true);
         CheckDlgButton (hwnd,IDC_OFAUTO    ,config->big_font.automatic);
         select_string  (hwnd,IDC_OFFONTNAME,config->big_font.name     );
         SetDlgItemText (hwnd,IDC_OFHEIGHT  ,TEXT(""));
         break;
  }
  EnableWindow (GetDlgItem(hwnd,IDC_OFFONTNAME),!IsDlgButtonChecked(hwnd,IDC_OFAUTO));
  check_height (hwnd);
  return;
}

//
//  Small utility routine to check the state of the height box.  If the current display font
//  is not a TrueType font then this will disable the window, otherwise it will enable the 
//  window.
//
static void check_height (HWND hwnd) {
  int   i;
  TCHAR buffer[SIZE_BUFFER];
  i = SendDlgItemMessage(hwnd,IDC_OFFONT,CB_GETCURSEL,0,0);
  SendDlgItemMessage (hwnd,IDC_OFFONTNAME,CB_GETLBTEXT,SendDlgItemMessage(hwnd,IDC_OFFONTNAME,CB_GETCURSEL,0,0),(LPARAM) buffer);
  EnableWindow (GetDlgItem(hwnd,IDC_OFHEIGHT),(lstrlen(buffer) >= 4) && stricmp(buffer+lstrlen(buffer)-4,TEXT(".f00")) && ((FONT_BIGFONT != i) && !IsDlgButtonChecked(hwnd,IDC_OFAUTO)));
  return;
}

//
//  Utility routine used to select only TrueType fonts for the font list.  In the case of 
//  PPC machines, any font is accepted because the PPC's don't support TT fonts.
//
//      lParam -- Passed indirectly via call to EnumFontFamilies() contains a pointer to the
//                dialog window so that messages can be sent to the controls
//
static int CALLBACK enum_fonts (ENUMLOGFONT *lpelf,NEWTEXTMETRIC *lpntm,int FontType,LPARAM lParam) {
#ifdef WINCE               // not all Win CE devices have TT fonts, so allow all fonts.
  SendDlgItemMessage ((HWND) lParam,IDC_OFASCIIFONT,CB_ADDSTRING,0,(LPARAM) lpelf->elfLogFont.lfFaceName);
#else
  if (FontType == TRUETYPE_FONTTYPE) {
    SendDlgItemMessage ((HWND) lParam,IDC_OFASCIIFONT,CB_ADDSTRING,0,(LPARAM) lpelf->elfLogFont.lfFaceName);
  }
#endif WINCE
  return (true);
} 
  
//
//  Utility routine used to locate all Japanese TrueType fonts, so they can be included in 
//  the print lists.
//
//      lParam -- Passed indirectly via call to EnumFontFamilies() contains a pointer to the
//                dialog window so that messages can be sent to the controls
//
static int CALLBACK enum_jfonts (ENUMLOGFONT *lpelf,NEWTEXTMETRIC *lpntm,int FontType,LPARAM lParam) {
  if (FontType != TRUETYPE_FONTTYPE) return (true);
  if (!config->all_fonts && (lpelf->elfLogFont.lfCharSet != SHIFTJIS_CHARSET)) return (true);
  SendDlgItemMessage ((HWND) lParam,IDC_OFFONTNAME,CB_ADDSTRING,0,(LPARAM) lpelf->elfLogFont.lfFaceName);
  return (true);
} 


//
//  Opens a particular ascii font.  This is used not only for this page, but in the
//  general setup of the options dialog.  This font is used to display the ASCII font
//  face.
//
//      face   -- Name of font to open.
//
//      RETURN -- Pointer to font or a NULL if the font could not be open.
//
static HFONT open_ascii (TCHAR *face) {
  LOGFONT    lf;
  memset (&lf,0,sizeof(lf));
  lstrcpy (lf.lfFaceName,face);
  lf.lfHeight = -sys_font.height;
  return (CreateFontIndirect(&lf));
}

//
//  Internal routine used to select an exect string from the font
//  list, this is necessary because the font selection can be wrong, 
//  Ariel Bold is listed in front of Ariel.
//
static void select_string (HWND hwnd,int id,TCHAR *string) {
  int i;
  i = SendDlgItemMessage(hwnd,id,CB_FINDSTRINGEXACT,0,(LPARAM) string);
  SendDlgItemMessage (hwnd,id,CB_SETCURSEL,i,0);
  return;
}

//
//  Internal routine to setup the fonts in the Japanese and printer font 
//  lists.  This routine was sepearated out so it can be called again if 
//  the user changes the state of the Show All Fonts button.
//
//      hwnd -- Pointer to dialog window.
//
static void setup_jfonts (HWND hwnd) {
  HDC             hdc;
  HANDLE          handle;
  WIN32_FIND_DATA data;
//
//  Reset lists to get an empty list.
//
  SendDlgItemMessage (hwnd,IDC_OFFONTNAME,CB_RESETCONTENT,0,0);
  handle = FindFirstFile(jfc_config.name(TEXT("*.f00")),&data);
  if (handle != INVALID_HANDLE_VALUE) {
    do {
      SendDlgItemMessage (hwnd,IDC_OFFONTNAME,CB_ADDSTRING,0,(LPARAM) data.cFileName);
    } while (FindNextFile(handle,&data));
    FindClose (handle);
  }
//
//  Process TrueType fonts
//
  hdc = GetDC(hwnd);
  EnumFontFamilies (hdc,NULL,(FONTENUMPROC) enum_jfonts,(LPARAM) hwnd);
  change_font (hwnd);
  return;
}

//
//  The Fonts/Display page.
//
#define NUMBER_FONTS    (sizeof(fonts)/sizeof(short))

static BOOL CALLBACK options_font (HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam) {
  static short fonts[] = { IDS_FONT_SYSTEM,IDS_FONT_KANA,IDS_FONT_KANJI,IDS_FONT_IMI,IDS_FONT_BIGFONT };
  switch (msg) {
    case WM_INITDIALOG: {
           HDC  hdc;
           int  i;
           SetDlgItemText     (hwnd,IDC_OFASCIINAME,config->ascii_font.name);
           SendDlgItemMessage (hwnd,IDC_OFASCIINAME,WM_SETFONT,(WPARAM) font,MAKELPARAM(true,0));
           SendDlgItemMessage (hwnd,IDC_OFASCIIFONT,CB_RESETCONTENT,0,0);
           CheckDlgButton     (hwnd,IDC_OFSHOWALL   ,config->all_fonts);
           CheckDlgButton     (hwnd,IDC_OFKANAFONT  ,config->kana_for_advanced);
//
//  Setup the font type list
//
           SendDlgItemMessage (hwnd,IDC_OFFONT,CB_RESETCONTENT,0,0);
           for (i = 0; i < NUMBER_FONTS; i++) SendDlgItemMessage (hwnd,IDC_OFFONT,CB_ADDSTRING,0,(LPARAM) get_string(fonts[i]));
           SendDlgItemMessage (hwnd,IDC_OFFONT,CB_SETCURSEL,FONT_KANA,0);
//
//  Setup the font list.
//
           current_font = FONT_NONE;
           hdc = GetDC(hwnd);
           SendDlgItemMessage (hwnd,IDC_OFFONTNAME,CB_RESETCONTENT,0,0);
           EnumFontFamilies   (hdc,NULL,(FONTENUMPROC) enum_fonts ,(LPARAM) hwnd);
           ReleaseDC          (hwnd,hdc);
           select_string      (hwnd,IDC_OFASCIIFONT  ,config->ascii_font.name);
           setup_jfonts       (hwnd);
         }
         return (true);
    case WM_COMMAND:
         switch (LOWORD(wParam)) {
           INPUT_CHECK (IDC_OFHEIGHT);
           case IDCANCEL:                       // Allow ESC abort
                DestroyWindow (hwnd);
                return (true);
//
//  Change to the font.
//
           case IDC_OFASCIIFONT:
                SendDlgItemMessage (hwnd,IDC_OFASCIIFONT,CB_GETLBTEXT,SendDlgItemMessage(hwnd,IDC_OFASCIIFONT,CB_GETCURSEL,0,0),(LPARAM) config->ascii_font.name);
                DeleteObject (font);
                font = open_ascii (config->ascii_font.name);
                SetDlgItemText     (hwnd,IDC_OFASCIINAME,config->ascii_font.name);
                SendDlgItemMessage (hwnd,IDC_OFASCIINAME,WM_SETFONT,(WPARAM) font,MAKELPARAM(true,0));
                return (true);
//
//  Working with the kanji fonts
//
           case IDC_OFAUTO:
           case IDC_OFFONT:
                change_font (hwnd);
                return (true);
           case IDC_OFFONTNAME:  
                check_height (hwnd);
                return (true);
//
//  Changes to the show all buttons will require that we save the current settings,
//  then reinitialize the fonts lists.
//                
           case IDC_OFSHOWALL:
                config->all_fonts = IsDlgButtonChecked(hwnd,IDC_OFSHOWALL);
                setup_jfonts (hwnd);
                return (true);
         }
         break;
    case WM_GETDLGVALUES:
         change_font (hwnd);                    // This forces a read of the font settings.
         config->kana_for_advanced = IsDlgButtonChecked(hwnd,IDC_OFKANAFONT);
         return (true);
  }
  return (false);
}

//...................................................................
//
//  Vocab page
//

//
//  The Vocab page
//
static BOOL CALLBACK options_vocab (HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam) {
  switch (msg) {
    case WM_INITDIALOG:
         CheckDlgButton (hwnd,IDC_OVKANJI       ,config->flash_kanji  );
         CheckDlgButton (hwnd,IDC_OVKANA        ,config->flash_kana   );
         CheckDlgButton (hwnd,IDC_OVASCII       ,config->flash_ascii  );
         CheckDlgButton (hwnd,IDC_OVKANJIALWAYS ,config->kanji_always );
         CheckDlgButton (hwnd,IDC_OVUNKNOWNKANJI,config->kanji_unknown);
         CheckDlgButton (hwnd,IDC_OVALLKNOWN    ,config->know_all_kanji);
         CheckDlgButton (hwnd,IDC_OVCOLORKANJI  ,config->knownkanji_colorkanji);
         goto set_knownkanji;
    case WM_COMMAND:
         switch (LOWORD(wParam)) {
set_knownkanji:;
           case IDC_OVKANJIALWAYS:
                EnableWindow (GetDlgItem(hwnd,IDC_OVUNKNOWNKANJI),!IsDlgButtonChecked(hwnd,IDC_OVKANJIALWAYS));
                return (true);
         }
         break;
    case WM_GETDLGVALUES:
         config->flash_kanji           = IsDlgButtonChecked(hwnd,IDC_OVKANJI       );
         config->flash_kana            = IsDlgButtonChecked(hwnd,IDC_OVKANA        );
         config->flash_ascii           = IsDlgButtonChecked(hwnd,IDC_OVASCII       );
         config->kanji_always          = IsDlgButtonChecked(hwnd,IDC_OVKANJIALWAYS );
         config->kanji_unknown         = IsDlgButtonChecked(hwnd,IDC_OVUNKNOWNKANJI);
         config->know_all_kanji        = IsDlgButtonChecked(hwnd,IDC_OVALLKNOWN    );
         config->knownkanji_colorkanji = IsDlgButtonChecked(hwnd,IDC_OVCOLORKANJI  );
         return (true);
  }
  return (false);
}

//...................................................................
//
//  Kanji page
//

//
//  The kanji page.
//
static BOOL CALLBACK options_kanji (HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam) {
  switch (msg) {
    case WM_INITDIALOG:
         CheckDlgButton (hwnd,IDC_OKKANJI      ,config->kanji_kanji      );
         CheckDlgButton (hwnd,IDC_OKKANA       ,config->kanji_kana       );
         CheckDlgButton (hwnd,IDC_OKASCII      ,config->kanji_ascii      );
         CheckDlgButton (hwnd,IDC_OKFULL       ,config->full_kanjicard   );
         CheckDlgButton (hwnd,IDC_OKAUTOEXPAND ,config->auto_expand      );
         CheckDlgButton (hwnd,IDC_OKAUTOCOLAPSE,config->auto_collapse    );
    case WM_GETDLGVALUES:
         config->kanji_kanji       = IsDlgButtonChecked(hwnd,IDC_OKKANJI      );
         config->kanji_kana        = IsDlgButtonChecked(hwnd,IDC_OKKANA       );
         config->kanji_ascii       = IsDlgButtonChecked(hwnd,IDC_OKASCII      );
         config->full_kanjicard    = IsDlgButtonChecked(hwnd,IDC_OKFULL       );
         config->auto_expand       = IsDlgButtonChecked(hwnd,IDC_OKAUTOEXPAND );
         config->auto_collapse     = IsDlgButtonChecked(hwnd,IDC_OKAUTOCOLAPSE);
         return (true);
  }
  return (false);
}

//...................................................................
//
//  Misc page
//

//
//  The misc page.
//
static BOOL CALLBACK options_misc (HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam) {
  switch (msg) {
    case WM_INITDIALOG:
         CheckDlgButton (hwnd,IDC_OMINFOTITLES,config->info_titles);
         CheckDlgButton (hwnd,IDC_OMINFOCOMPRESS,config->info_compress);
         CheckDlgButton (hwnd,IDC_OMCOLORKANJI,config->colorkanji_mode);
         CheckDlgButton (hwnd,(config->colorkanji_mode == COLORKANJI_MATCH) ? IDC_OMINLIST : IDC_OMNOTINLIST,true);
         SendDlgItemMessage (hwnd,IDC_OMKANJIBOX,WMU_SETWINDOWVALUE,0,config->colorkanji_color);
         SendDlgItemMessage (hwnd,IDC_OMTITLEBOX,WMU_SETWINDOWVALUE,0,config->info_color);
         goto set_colorkanji;
    case WM_COMMAND:
         switch (LOWORD(wParam)) {
           case IDC_OMCOLORKANJI:
set_colorkanji:;
                lParam = IsDlgButtonChecked(hwnd,IDC_OMCOLORKANJI);
                EnableWindow (GetDlgItem(hwnd,IDC_OMINLIST    ),lParam);
                EnableWindow (GetDlgItem(hwnd,IDC_OMNOTINLIST ),lParam);
                EnableWindow (GetDlgItem(hwnd,IDC_OMKANJICOLOR),lParam);
                return (true);
           case IDC_OMKANJIBOX:
           case IDC_OMKANJICOLOR:
                config->colorkanji_color = get_color (hwnd,config->colorkanji_color);
                SendDlgItemMessage (hwnd,IDC_OMKANJIBOX,WMU_SETWINDOWVALUE,0,config->colorkanji_color);
                return (true);
           case IDC_OMTITLEBOX:
           case IDC_OMTITLECOLOR:
                config->info_color = get_color(hwnd,config->info_color);
                SendDlgItemMessage (hwnd,IDC_OMTITLEBOX,WMU_SETWINDOWVALUE,0,config->info_color);
                return (true);
         }
         break;
    case WM_GETDLGVALUES:
         config->info_titles   = IsDlgButtonChecked (hwnd,IDC_OMINFOTITLES);
         config->info_compress = IsDlgButtonChecked (hwnd,IDC_OMINFOCOMPRESS);
         if (!IsDlgButtonChecked(hwnd,IDC_OMCOLORKANJI)) config->colorkanji_mode = COLORKANJI_OFF;
           else config->colorkanji_mode = IsDlgButtonChecked(hwnd,IDC_OMINLIST) ? COLORKANJI_MATCH : COLORKANJI_NOMATCH;
         return (true);
  }
  return (false);
}

//...................................................................
//
//  Review page
//

//
//  This rotuine is called to read and to change the values in the review mode list.  
//  During setup this routine is called with init set to true.  This causees the values 
//  to simply be intialized, and no other action taken. 
//
//  The rotuine keeps track of the current set of values, and always reads the old values 
//  out then set the new values.  
//
//  When complete, this routine should be called once ot read the values out.
//
//      hwnd -- Dialog box pointer.
//      init -- Set to true for the first call only.  This causes the values not to be
//              read, but only to be set.
//
static void set_type (HWND hwnd,int init) {
  int i;
  static short old_type;
  if (!init) {
    for (i = 0; i < 9; i++) config->flashkanji[old_type+i] = get_int(hwnd,IDC_OR8+i,0,100,config->flashkanji[old_type+i]);
  }
  old_type = 10*sizeof(byte)*SendDlgItemMessage(hwnd,IDC_ORTYPE,CB_GETCURSEL,0,0);
  for (i = 0; i < 9; i++) SetDlgItemInt (hwnd,IDC_OR8+i,config->flashkanji[old_type+i],false);
  return;
}

//
//  The review page.
//
static BOOL CALLBACK options_review (HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam) {
  switch (msg) {
    case WM_INITDIALOG:
         SendDlgItemMessage (hwnd,IDC_ORTYPE,CB_ADDSTRING,0,(LPARAM) get_string(IDS_OR_VOCABKANJI  ));
         SendDlgItemMessage (hwnd,IDC_ORTYPE,CB_ADDSTRING,0,(LPARAM) get_string(IDS_OR_VOCABKANA   ));
         SendDlgItemMessage (hwnd,IDC_ORTYPE,CB_ADDSTRING,0,(LPARAM) get_string(IDS_OR_VOCABMEANING));
         SendDlgItemMessage (hwnd,IDC_ORTYPE,CB_ADDSTRING,0,(LPARAM) get_string(IDS_OR_KANJIKANJI  ));
         SendDlgItemMessage (hwnd,IDC_ORTYPE,CB_ADDSTRING,0,(LPARAM) get_string(IDS_OR_KANJIREAD   ));
         SendDlgItemMessage (hwnd,IDC_ORTYPE,CB_ADDSTRING,0,(LPARAM) get_string(IDS_OR_KANJIMEANING));
         SendDlgItemMessage (hwnd,IDC_ORTYPE,CB_SETCURSEL,0,0);
         set_type (hwnd,true);
         return   (true);
    case WM_COMMAND:
         switch (LOWORD(wParam)) {
           INPUT_CHECK (IDC_OR8);
           INPUT_CHECK (IDC_OR7);
           INPUT_CHECK (IDC_OR6);
           INPUT_CHECK (IDC_OR5);
           INPUT_CHECK (IDC_OR4);
           INPUT_CHECK (IDC_OR3);
           INPUT_CHECK (IDC_OR2);
           INPUT_CHECK (IDC_OR1);
           INPUT_CHECK (IDC_OR0);
           case IDC_ORTYPE:
                set_type (hwnd,false);
                return   (true);
         }
         break;
    case WM_GETDLGVALUES:
         set_type (hwnd,false);
         return   (true);
  }
  return (false);
}

//...................................................................
//
//  Advanced page
//

//
//  Advanced page
//
static BOOL CALLBACK options_advanced (HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam) {
  switch (msg) {
    case WM_INITDIALOG:
         CheckDlgButton (hwnd,IDC_OAINSTALL     ,config->install);
         CheckDlgButton (hwnd,IDC_OACACHESYSTEM ,config->cache_systemfont);
         CheckDlgButton (hwnd,IDC_OACACHEDISPLAY,config->cache_displayfont);
         CheckDlgButton (hwnd,IDC_OACACHEINFO   ,config->cache_info);
         SetDlgItemInt  (hwnd,IDC_OAFONTCACHE   ,config->font_cache,false);
         SetDlgItemInt  (hwnd,IDC_OASUBFILES    ,config->max_subfiles,false);
         return (true);
#ifdef WINCE_PPC
#ifndef _ARM_
    case WM_COMMAND:
         switch (LOWORD(wParam)) {
           INPUT_CHECK (IDC_OAFONTCACHE);
         }
         break;
#endif _ARM_
#endif WINCE_PPC
    case WM_GETDLGVALUES:
         config->install           = IsDlgButtonChecked(hwnd,IDC_OAINSTALL);
         config->cache_systemfont  = IsDlgButtonChecked(hwnd,IDC_OACACHESYSTEM);
         config->cache_displayfont = IsDlgButtonChecked(hwnd,IDC_OACACHEDISPLAY);
         config->cache_info        = IsDlgButtonChecked(hwnd,IDC_OACACHEINFO);
         config->font_cache        = get_int(hwnd,IDC_OAFONTCACHE,100,7000,config->font_cache);
         config->max_subfiles      = get_int(hwnd,IDC_OASUBFILES ,10,INT_MAX,config->max_subfiles);
         return (true);
  }
  return (false);
}

//
//  This is not a page handler, but rather a handler for the main dialog buttons.
//
static BOOL CALLBACK options_main (HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam) {
  HWND      tab;                // Window pointer for page and tab control.
  TabSetup *setup;              // Pointer to setup structure.
  TC_ITEM   item;               // Item structure to access data in the tab control.
  int       i;              
  switch (msg) {
    case WM_COMMAND:
         switch (LOWORD(wParam)) {
           case IDC_TABDEFAULT:                                         // Restore default configuration.
                DeleteObject (font);                                    // Restore the configuration.
                font      = open_ascii(jfc_config.cfg.ascii_font.name);
                *config   = default_config;
                setup     = (TabSetup *) GetWindowLong(hwnd,DWL_USER);  // Reinitialize each page.
                tab       = GetDlgItem(hwnd,IDC_TABCONTROL);
                item.mask = TCIF_PARAM;
                for (i = 0; i < setup->count; i++) {
                  TabCtrl_GetItem (tab,i,&item);
                  SendMessage ((HWND) (item.lParam),WM_INITDIALOG,0,0);
                }
                return (true);
         }
         break;
  }
  return (false);
}

//-------------------------------------------------------------------
//
//  Begin Class JFC_config.
//
//  Simple class to handle reading and writing the intialization file
//
//  The structure of the configuation file is as follows:
//
//      binary struct cfg -- Contains most of the options.  Note the 
//                           magic field indicates the config file version.
//                           As options are added this field is incremented
//                           and JWPce will reject old configurations.
//      recent files      -- Next are 9 null terminated strings.  These
//                           are file names for the recent files list.
//                           Even if there are less than 9 files in the 
//                           list, 9 file names will be here.  The non-
//                           existant names will be written as zeros.
//      working directory -- Stored working directory.
//      load files        -- The remainder of the list is a number of 
//                           load files.  These are null termianted 
//                           strings of files to load on startup.
//

class JFC_config jfc_config;

//
//  Constructor.
//
JFC_config::JFC_config () {
  TCHAR buf[SIZE_BUFFER];
  ok  = false;              // Disk configuration is not valid.
  cfg = default_config;     // Start with default configuration.
//
//  Windows CE startup, locate the executalbe file to find the config
//  file location.
//
#ifdef WINCE
  GetModuleFileName (instance,buf,SIZE_BUFFER);
  GetFullPathName (buf,SIZE_BUFFER,buffer,&ptr);
//
//  Windows NT/95 startup.  This startup loads the command line and 
//  reads the first command parameter to determine if the Network 
//  option is being loaded.
//
#else WINCE
  TCHAR *p;
  p = get_parameter (buf,GetCommandLine());
  GetFullPathName (buf,sizeof(buffer),buffer,&ptr);
#endif WINCE
  return;
}

//
//  Set the default configuaration.  This is called as a responce to the default 
//  configuration menu command.
//
void JFC_config::def () {
  set (&default_config);
  return;
}

//
//  Geneate a file name for operations.
//
//      file   -- Name of file.
//  
//      RETURN -- Pointer to buffer containning full path to file.
//
TCHAR *JFC_config::name (TCHAR *file) {
  lstrcpy (ptr,file);
  return  (buffer);
}

//
//  Open a file located in the program directory.
//
//      filename -- Base name of file to open.
//      mode     -- Open mode (OPEN_READ or OPEN_WRITE or OPEN_APPEND).
//
//      RETURN   -- HANDLE for a file.
//
HANDLE JFC_config::open (TCHAR *filename,int mode) {
  TCHAR *ptr;
  int    cmode;
  HANDLE handle;
#ifndef WINCE
  UINT   error_mode;
  error_mode = SetErrorMode(SEM_FAILCRITICALERRORS);    // Diable errors incase we are using a floppy
#endif WINCE
  ptr = name(filename);
  if (mode == OPEN_READ) {
    handle = CreateFile(ptr,GENERIC_READ ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
  }
  else {
    if (mode == OPEN_APPEND) cmode = OPEN_ALWAYS; else cmode = CREATE_ALWAYS;
    handle = CreateFile(ptr,GENERIC_WRITE,0              ,NULL,cmode        ,FILE_ATTRIBUTE_NORMAL,NULL);
  }
#ifndef WINCE
  SetErrorMode (error_mode);                            // Re-enable error handling.
#endif
  return (handle);
}

//
//  This routine does the options dialog box.
//
void JFC_config::options () {
  int        i;
  struct cfg new_config;
//
//  Static data for the page dialog.
//  
  static TabPage pages[] = {
    { IDS_OPT_GENERAL ,IDD_OPTGENERAL ,(DLGPROC) options_general ,IDH_OPTIONS_GENERAL  },
    { IDS_OPT_CONTROLS,IDD_OPTCONTROLS,(DLGPROC) options_controls,IDH_OPTIONS_CONTROLS },
    { IDS_OPT_CARDS   ,IDD_OPTCARDS   ,(DLGPROC) options_cards   ,IDH_OPTIONS_CARDS    },
    { IDS_OPT_FONT    ,IDD_OPTFONT    ,(DLGPROC) options_font    ,IDH_OPTIONS_FONT     },
    { IDS_OPT_VOCAB   ,IDD_OPTVOCAB   ,(DLGPROC) options_vocab   ,IDH_OPTIONS_VOCAB    },
    { IDS_OPT_KANJI   ,IDD_OPTKANJI   ,(DLGPROC) options_kanji   ,IDH_OPTIONS_KANJI    },
    { IDS_OPT_MISC    ,IDD_OPTMISC    ,(DLGPROC) options_misc    ,IDH_OPTIONS_MISC     },
    { IDS_OPT_REVIEW  ,IDD_OPTREVIEW  ,(DLGPROC) options_review  ,IDH_OPTIONS_REVIEW   },
    { IDS_OPT_ADVANCED,IDD_OPTADVANCED,(DLGPROC) options_advanced,IDH_OPTIONS_ADVANCED },
  };
  static TabSetup setup = { pages,sizeof(pages)/sizeof(TabPage),0,(DLGPROC) options_main };
//
//  Intialize the variables and call the dialog box.
//
  new_config = cfg;
  config     = &new_config;
  font       = open_ascii(cfg.ascii_font.name);
  i = TabDialog(IDD_OPTIONS,&setup);
  DeleteObject (font);
//
//  Change the parameters.
//
  if (i) set (&new_config);
  return;
}

//
//  Read configuration file.
//
//      RETURN -- Non-zero indicates an allocation error.
//
int JFC_config::read () {
  int    i;
  HANDLE hfile;
  unsigned long done;
  if (INVALID_HANDLE_VALUE != (hfile = open(NAME_CONFIG,OPEN_READ))) {
    if (!ReadFile(hfile,&cfg,sizeof(cfg),&done,NULL) || (cfg.magic != CONFIG_MAGIC)) goto ConfigError;
    ok   = true;
    i    = GetFileSize(hfile,NULL)-sizeof(cfg);
    load = (TCHAR *) calloc(i+24,1);        // Enough extra characters to handle UNICODE
    ReadFile(hfile,load,i,&done,NULL);
    goto AdjustConfig; 
  }
ConfigError:
  cfg = default_config;
  write ();
  load = NULL;
  ErrorMessage (false,IDS_CONFIG_LOAD,name());
AdjustConfig:
  CloseHandle (hfile);
#ifndef WINCE
  if (!cfg.usedims) cfg.x = cfg.y = cfg.xs = cfg.ys = CW_USEDEFAULT;
#endif WINCE
  return (false);
}

//
//  This routine is called to initialize the set values to a new 
//  configuration.  All the updating that is necessary to perform the 
//  update is performed.  This should not be used for initializing 
//  the configuration, but rather for initalizaing the configuration 
//  after a change has been made.
//
//      new_config -- New configuration.
//
void JFC_config::set (struct cfg *new_config) {
//
//  Save some paraemters then transfer the whole config structure.
//
  cfg       = *new_config;              // Do assign.
  cfg.magic = CONFIG_MAGIC;             // Make sure magic is set correctly
//
//  Setup the display and all the other features.
//
  initialize_fonts    ();
  jfc_stat.initialize (NULL);
  jfc_stat.adjust     ();
  jfc_file->reinit    (true);
  window_title        ();
  return;
}

//
//  Write configuration file.
//
//  This routine writes the configuration structure to the config file.
//  It does not write the file names, write_files() does that.  This 
//  routine must be called before write_files(), since this rotuine 
//  destroys the contents of the configuration file.
//
void JFC_config::write () {
  HANDLE          hfile;
  int             err;
  unsigned long   done;
  if (!main_window || !ptr) return;
#ifndef WINCE
  WINDOWPLACEMENT placement;
  placement.length = sizeof(placement);
  GetWindowPlacement (main_window,&placement);
  cfg.x  = placement.rcNormalPosition.left;
  cfg.y  = placement.rcNormalPosition.top;
  cfg.xs = (placement.rcNormalPosition.right -placement.rcNormalPosition.left+1);
  cfg.ys = (placement.rcNormalPosition.bottom-placement.rcNormalPosition.top +1);
  cfg.maximize = (byte) ((placement.showCmd == SW_MAXIMIZE) || ((placement.showCmd == SW_SHOWMINIMIZED) && (placement.flags == WPF_RESTORETOMAXIMIZED)));
#endif WINCE
  hfile = open(NAME_CONFIG,OPEN_APPEND);
  if (INVALID_HANDLE_VALUE != hfile) {
    err = !WriteFile(hfile,&cfg,sizeof(cfg),&done,NULL);
    ok = true;
    CloseHandle  (hfile);
    if (!err) return;
  }
  ErrorMessage (true,IDS_CONFIG_SAVE,name());
  return;
}

//
//  Write file names to the configuration file.
//
//  This routine writes the file names to the confifugation file.
//  Basically this skips to the end of the fixed data structure then
//  writes 9 recent files, and then writes the names of the currently 
//  open files.
//
#define STRINGSIZE(x)   (sizeof(TCHAR)*(x))

void JFC_config::write_files () {
  int       i;
  HANDLE    hfile;
  TCHAR     buffer[SIZE_BUFFER];
  unsigned long done;
//
//  Open file.  Cannot use JWP_config::open() because we want to maintain
//  the contents of the file and open() destroys the previous file.
//
  hfile = open(NAME_CONFIG,OPEN_APPEND);
  if (INVALID_HANDLE_VALUE == hfile) {
    ErrorMessage (true,IDS_CONFIG_WRITE,name());
    return;
  }
  SetFilePointer (hfile,sizeof(cfg),NULL,FILE_BEGIN);   // Move to end of fixed strucutre.
  SetEndOfFile   (hfile);                               // Truncate file here.
  for (i = 8; i >= 0; i--) {                            // Write recent files.
    if (!get_menudata(hmenu,IDM_FILE_FILES_BASE+i,false,buffer)) buffer[3] = 0; 
    WriteFile (hfile,buffer+3,STRINGSIZE(lstrlen(buffer+3)+1),&done,NULL);
  }
  GetCurrentDirectory (SIZE_BUFFER,buffer);             // Save current directory.
  WriteFile (hfile,buffer,STRINGSIZE(lstrlen(buffer)+1),&done,NULL);
//
//  Write the currently loaded file.  Since we are not keeping the file 
//
  if (jfc_file) WriteFile (hfile,jfc_file->filename,STRINGSIZE(lstrlen(jfc_file->filename)+1),&done,NULL);
  CloseHandle (hfile);
  return;
}

//
//  End Class JFC_config.
//
//-------------------------------------------------------------------

//
//  Initializes the class, and actually enerates the window.
//
int initialize_config (WNDCLASS *wclass) {
  wclass->lpfnWndProc   = JFC_color_proc;           // Color button class for dialog boxes.
  wclass->lpszClassName = TEXT("JFC-Color");
  if (!RegisterClass(wclass)) return (true);
  return (false);
}









