//-------------------------------------------------------------------//
//                                                                   //
//  JFC Copyright (C) Glenn Rosenthal, 1999,2000,2001.               //
//  All rights reserved.                                             //
//                                                                   //
//-------------------------------------------------------------------//

#include "jfc_main.h"
#include "jfc_conf.h"
#include "jfc_edit.h"
#include "jfc_dict.h"
#include "jfc_file.h"
#include "jfc_font.h"
#include "jfc_help.h"
#include "jfc_info.h"
#include "jfc_inst.h"
#include "jfc_jisc.h"
#include "jfc_misc.h"
#include "jfc_stat.h"
#include "jfc_wind.h"

#include <objbase.h>
#include <shlobj.h>
#include <commctrl.h>

#if (defined(WINCE_PPC) && !defined(_ARM_))
  #include <Aygshell.h>
#endif 

//-------------------------------------------------------------------
//
//  These codes define the values required for a language DLL.  These values are combined
//  the version ID to build a full value.  These letter codes indicate the type of 
//  platform that the DLL is intended to support.
//
#if   defined(WINCE_PPC)
  #define LANGUAGE_CODE 'P'
#elif defined(WINCE_HPC)
  #define LANGUAGE_CODE 'H'
#else
  #define LANGUAGE_CODE 'W'     
#endif

//-------------------------------------------------------------------
//
//  Static data.
//

static HACCEL haccel = null;        // The accelerator table for the menu.

#ifdef WINCE
static TCHAR *tip_data   = NULL;    // Data use for the tool-tips
#endif WINCE

//-------------------------------------------------------------------
//
//  Exported data.
//

HMENU     hmenu    = null;      // The main menu.
HINSTANCE instance = null;      // Our instance.
HINSTANCE language = null;      // Localaization libary instance.

#ifdef WINCE
HWND  command_bar = null;       // The Windows CE command bar.
#endif WINCE

//-------------------------------------------------------------------
//
//  Static procedures.
//

static int    open_files    (TCHAR *command);           // Intialization rotuine to open all files.
static void   terminate     (int    format,...);        // Bail-out routine used if cannot intialize.

//
//  This routine is called only once, during startup to open all files
//  the user has requested.  This includes files from the command line, 
//  and previously loaded files, if that option is set.
//
//  The reason this is separite from the main windows procedure is so 
//  the stack space required does not get allocated on that routine's 
//  stack, since such space would remain dead and used for the remainder
//  of the program (in CE space, the two buffers use 2k).
//
//      command -- Command line passed from the user.
//
//      RETURN  -- A non-zero return value indicates a load error,
//                 and no files could be open.  This is probobly a 
//                 shortage of memory.
//
static int open_files (TCHAR *command) {
  int    i;
  TCHAR *ptr;
//
//  Process recent files, and reload files.
//
//  When the main configuration was loaded, the extneded contents of 
//  the config file (stuff after the fixed structure) were loaded into
//  the JWP_config::load variable, so we can access them then destroy 
//  them.
//
  if (jfc_config.load) {
    for (ptr = jfc_config.load, i = 0; i < 9; i++, ptr += lstrlen(ptr)+1) {
      if (*ptr) recent_files (ptr);
    }
    if (jfc_config.cfg.reload_files) {
#ifdef WINCE
      set_currentdir (ptr,false);
#else
      if (*ptr) SetCurrentDirectory (ptr);
#endif
      ptr += lstrlen(ptr)+1;
      if (*ptr) {
        if (FileExists(ptr)) open_file (ptr);
      }
    }
    free (jfc_config.load);
    jfc_config.load = NULL;
  }
//
//  Process command line arguments.
//
  ptr = command;
  if (ptr && *ptr) {
    if (*ptr == '"') {
      ptr++;
      if (ptr[lstrlen(ptr)-1] == '"') ptr[lstrlen(ptr)-1] = 0;
    }
    jfc_file->close (false);
    open_file (ptr);
  }
  return (false);
}

//
//  If things work correctly, this rotuine should never get called.
//  This is a panic stop.  It is only used until the system is started.
//  If any module returns a failure during startup this is called and
//  we stop.
//
//  The configuration is not saved here because we don't want to trash
//  the user's configuration on a failture to stratup.
//
//      format... -- printf type format string containning the termination
//                   message.
//
static void terminate (int format,...) {
  TCHAR buffer[SIZE_BUFFER],string[SIZE_BUFFER];
  va_list argptr;
  va_start (argptr,format);
  if (format) {
    GET_STRING (string,format);
    wvsprintf  (buffer,string,argptr);
    MessageBox (NULL,buffer,get_string(IDS_TERMINAL),MB_OK | MB_ICONWARNING);
  }
  if (main_window) DestroyWindow (main_window);
  CoUninitialize ();
  ExitThread (format ? 0 : 1);
}

//-------------------------------------------------------------------
//
//  Main windows rotuine.  WinMain and the main window procedures.
//

#define WINCLASS_MAIN   TEXT("JFC-Main")    // Main window class.

HWND main_window = NULL;                    // THE WINDOW!

static LRESULT CALLBACK winproc_main (HWND hwnd,UINT iMsg,WPARAM wParam,LPARAM lParam);

//
//  Main window procedure.  Menus, junk, etc.
//
static LRESULT CALLBACK winproc_main (HWND hwnd,UINT iMsg,WPARAM wParam,LPARAM lParam) {
//  int       ctrl,shift;

  switch (iMsg) {
//
//  Standard, when the user moves, or changes size we adjust the windows.
//
    case WM_SIZE:
    case WM_MOVE:
#ifdef WINCE
         jfc_stat.adjust ();
         jfc_window.adjust (-1);
#else WINCE
         if (!IsIconic(hwnd)) {         // Don't mess with window when it is minimized!
           jfc_stat.adjust ();
           jfc_window.adjust (-1);
         }
#endif WINCE
         return (0);
//
//  Timer messages
//
    case WM_TIMER:
         KillTimer (main_window,TIME_SHOW);             // These are necessary to handle changing the time options.
         KillTimer (main_window,TIME_NEXT);
         switch (wParam) {
           case TIME_SHOW:
                jfc_file->do_menu (IDM_CARD_SHOW);
                break;
           case TIME_NEXT:
                jfc_file->do_menu (IDM_CARD_WRONG);
                break;
         }
         return (0);
//
//  The Activate messages are used to shutdown the timer system when the 
//  user deactivates the main window, or when the user is in a dialog box.
//
//  If the timer options are on and there are no cards, this generates an
//  extra timing cycle, but that cycle stops after one cycle so this is not
//  too bad.
//
    case WM_ACTIVATE:
         if (!jfc_config.cfg.time_cards || !jfc_file) return (0);
         if ((wParam != WA_INACTIVE) && jfc_config.cfg.time_show) SetTimer (main_window,TIME_SHOW,jfc_config.cfg.time_show,NULL);
           else {
             KillTimer (main_window,TIME_SHOW);
             KillTimer (main_window,TIME_NEXT);
           }
         return (0);
//
//  On set focus to the main window, activate the first text input window.
//  This picks up the case of coming back from a dialog box, etc.
//
    case WM_SETFOCUS:
         if (jfc_window.number_windows) jfc_window.set (jfc_window.last_id,false);
         return (0);
//
//  Intertupt close so we can put up an Are You Sure? dialog.
//
    case WM_CLOSE:
         jfc_file->do_menu (IDM_FILE_EXIT);
         return (0);
//
//  Destroy means we are about to die, so clean up.
//
    case WM_DESTROY:
         jfc_config.done ();            // Save user configuration
#ifdef WINCE
         CommandBar_Destroy (command_bar);
         if (tip_data) free (tip_data);
#else  WINCE
         DeleteObject    (hmenu);       // Free lots of memory.
#endif WINCE
         DeleteObject    (haccel);
         free_fonts      (true);        // Close open fonts
         free_info       ();            // Free any info resource.
         free_dictionary ();            // Close user dicitonary.
         if (language && (language != instance)) FreeLibrary (language);
         PostQuitMessage (0);           // saiyanara (I hate romaji!!!)
         return (0);
//
//  Support for drag and drop:
//
#ifndef WINCE
    case WM_DROPFILES: {
           HDROP drop = (HDROP) wParam;
           TCHAR buffer[SIZE_BUFFER];
           if (DragQueryFile(drop,0,buffer,SIZE_BUFFER) > 0) {
             if (!jfc_file->close(true)) open_file (buffer);
           }
           DragFinish (drop);
           return (0);
         }
#endif WINCE
//
//  The menu commands.
//
    case WM_COMMAND:
         jfc_file->do_menu (wParam);
         return (0);
//
//  Implement help
//
    case WM_HELP:
         do_help (main_window,0);
         return (0);
  }
  return (DefWindowProc(hwnd,iMsg,wParam,lParam));
}

//
//  End Main Window routine.
//
//-------------------------------------------------------------------

//-------------------------------------------------------------------
//
//  Main executable module for JFC.
//

//
//  Windows main routine.
//
int WINAPI WinMain (HINSTANCE hInstance,HINSTANCE hPrevInstance,TCHAR *szCmdLine,int iCmdShow) {
  MSG      msg;
  WNDCLASS wndclass,*wclass;
//
//  For PPC's check to see if another version of JFC is running.
//
#ifdef WINCE_PPC
  HANDLE mutex;
  mutex = CreateMutex(NULL,false,TEXT("JFC-Mutex"));
  if (ERROR_ALREADY_EXISTS == GetLastError()) {
    if (main_window = FindWindow(WINCLASS_MAIN,NULL)) {
      CloseHandle (mutex);
      SetForegroundWindow (main_window);
    }
    return (0);
  }
#endif WINCE_PPC
//
//  Activate the COM libararies
//
#ifdef WINCE
  if (S_OK != CoInitializeEx(NULL,COINIT_MULTITHREADED)) terminate (IDS_TERM_COM);
#else  wince
  if (S_OK != CoInitialize(NULL)) terminate (IDS_TERM_COM);
#endif WINCE
//
//  Support for international versions.  Open the language DLL and see if we can read it.
//
  instance = hInstance;
  if (!(language = LoadLibrary(jfc_config.name(TEXT("jfc_lang.dll"))))) language = instance;
    else {
      TCHAR *ptr = get_string(IDS_LANG_ID);
      if ((ptr[0] != LANGUAGE_CODE) || strnicmp(ptr+1,VERSION_STRING,4)) {
        FreeLibrary (language);
        language = instance;
        JMessageBox (NULL,IDS_LANG_ERRORTEXT,IDS_LANG_ERRORTITLE,MB_OK | MB_ICONERROR,jfc_config.name());
      }
    }  
//
//  Do some initialization.
//
#ifndef WINCE
  if (!(hmenu  = LoadMenu(language,MAKEINTRESOURCE(IDR_MAINMENU)))) terminate (IDS_TERM_MENU);
#endif WINCE
  if (!(haccel = LoadAccelerators(instance,MAKEINTRESOURCE(IDR_ACCELERATOR)))) terminate (IDS_TERM_ACCELERATORS);
  if (!hPrevInstance) {
    wclass = &wndclass;
    memset (&wndclass,0,sizeof(wndclass));
//  wndclass.style         = CS_HREDRAW | CS_VREDRAW;
    wndclass.lpfnWndProc   = winproc_main;
//  wndclass.cbClsExtra    = 0;
    wndclass.cbWndExtra    = 4;                     // All windows carry 4 extra bytes for our data.
    wndclass.hInstance     = instance;
    wndclass.hIcon         = LoadIcon (instance,MAKEINTRESOURCE(IDI_MAINICON));
    wndclass.hbrBackground = (HBRUSH) (COLOR_MENU+1);
//  wndclass.lpszMenuName  = NULL;
    wndclass.lpszClassName = WINCLASS_MAIN;
#ifndef WINCE
    wndclass.hCursor       = LoadCursor (NULL,IDC_ARROW);
#endif WINCE
    if (!RegisterClass(&wndclass)) terminate (IDS_TERM_REGCLASS);
  }
  else {
    wclass = NULL;
  }
//
//  Setup default values for the current directoy.  This will get 
//  overwritten when the user changes things.
//
  TCHAR buffer[SIZE_BUFFER];
#ifdef WINCE
  set_currentdir      (get_folder(CSIDL_PERSONAL,buffer),false);
#else  WINCE
  SetCurrentDirectory (get_folder(CSIDL_PERSONAL,buffer));
#endif WINCE
//
//  Read configuration.
//
  if (jfc_config.read()) terminate (IDS_TERM_CONFIG);
//
//  Intialize main window.
//
#define NUMBER_TIPS 5

#ifdef WINCE
  main_window = CreateWindowEx(WS_EX_CONTEXTHELP,WINCLASS_MAIN,WINCLASS_MAIN,WS_VISIBLE,0,0,CW_USEDEFAULT,CW_USEDEFAULT,NULL,NULL,instance,NULL);
  ShowWindow   (main_window,SW_SHOW);
  UpdateWindow (main_window);
  command_bar = CommandBar_Create(instance,main_window,1);
  CommandBar_InsertMenubar (command_bar,instance,IDR_MAINMENU,0);
static TBBUTTON buttons[] = {
  { 0         ,0             ,TBSTATE_ENABLED,TBSTYLE_BUTTON | TBSTYLE_SEP     ,0,-1},
  { STD_FIND  ,IDM_CARD_SHOW ,TBSTATE_ENABLED,TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE,0,-1},
  { 15        ,IDM_CARD_RIGHT,TBSTATE_ENABLED,TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE,0,-1},
  { STD_DELETE,IDM_CARD_WRONG,TBSTATE_ENABLED,TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE,0,-1},
};
  static TCHAR *tips[NUMBER_TIPS+1] = { TEXT(""),NULL };        // Tool tips array, filled out at startup (CE requires static).
//
//  Setup the tool tips.
//
  TCHAR *ptr;
  int    i = 1;                                                     // Set this here because we use it below
  ptr = get_string(IDS_TOOL_TIPS);                                  // Get tool tips coded data.
  if (tip_data = (TCHAR *) malloc(sizeof(TCHAR)*lstrlen(ptr))) {    // Allocate storage.  If not possible we just forget the tips.
    lstrcpy (tip_data,ptr);                                         // Copy tips to the data array.
    for (ptr = tip_data; i <= NUMBER_TIPS; i++) {                   // Move through array.
      tips[i] = ptr;                                                // Save pointer
      while (*ptr && (*ptr != '\t')) ptr++;                         // Find next tab (marks end of tip)
      if (!*ptr) break;                                             // Off the end of the 
      *ptr = 0;                                                     // Change tab to end of string
      ptr++;                                                        // On to next tip.
    }
  }
//
//  Setup command bar.
//
  CommandBar_AddBitmap     (command_bar,HINST_COMMCTRL,IDB_STD_SMALL_COLOR,15,0,0);
//  CommandBar_AddBitmap     (command_bar,instance,IDB_SHOW ,1,0,0);
  CommandBar_AddBitmap     (command_bar,instance,IDB_RIGHT,1,0,0);
//  CommandBar_AddBitmap     (command_bar,instance,IDB_WRONG,1,0,0);
  CommandBar_AddButtons    (command_bar,sizeof(buttons)/sizeof(TBBUTTON),buttons);
  CommandBar_AddAdornments (command_bar,CMDBAR_HELP,0);
  CommandBar_AddToolTips   (command_bar,i+1,tips);
  hmenu = CommandBar_GetMenu(command_bar,0);
  SetForegroundWindow (main_window);                // Necessary because CE does not restore JWPce to top after configuration message!
#else WINCE
  if (!(main_window = CreateWindowEx(WS_EX_ACCEPTFILES | WS_EX_CONTEXTHELP,WINCLASS_MAIN,TEXT("JFC-Main Window"),WS_OVERLAPPEDWINDOW,jfc_config.cfg.x,jfc_config.cfg.y,jfc_config.cfg.xs,jfc_config.cfg.ys,NULL,NULL,instance,NULL))) terminate (IDS_TERM_MAINWINDOW);
  if ((iCmdShow != SW_SHOWMINIMIZED) && jfc_config.cfg.maximize && jfc_config.cfg.usedims) iCmdShow = SW_SHOWMAXIMIZED;
  ShowWindow   (main_window,iCmdShow);
  UpdateWindow (main_window);
  SetMenu      (main_window,hmenu);
#endif WINCE
  window_title   ();
  EnableMenuItem (hmenu,IDM_FILE_CLOSE ,MF_BYCOMMAND | MF_GRAYED);
  EnableMenuItem (hmenu,IDM_FILE_REOPEN,MF_BYCOMMAND | MF_GRAYED);
  do_install     (false);
//
//  Intialize everybody else.
//
  if (initialize_fonts())            terminate (IDS_TERM_INITALIZE,get_string(IDS_TERM_FONTS       ));
  if (jfc_stat.initialize(wclass))   terminate (IDS_TERM_INITALIZE,get_string(IDS_TERM_STATUSBAR   ));
  if (initialize_edit  (wclass))     terminate (IDS_TERM_INITALIZE,get_string(IDS_TERM_LISTCONTROLS));
  if (initialize_config(wclass))     terminate (IDS_TERM_INITALIZE,get_string(IDS_TERM_OPTCONTROLS ));
  if (initialize_info  (wclass))     terminate (IDS_TERM_INITALIZE,get_string(IDS_TERM_INFOCLASS   ));
  if (jfc_window.initialize())       terminate (IDS_TERM_INITALIZE,get_string(IDS_TERM_WINDOWLIST  ));
  initialize_cp ();
//
//  Initialize the random number generator.
//
  SYSTEMTIME st;
  GetSystemTime (&st);
  srand (st.wMinute);
//
//  Open files for the user.
//
  if (open_files(szCmdLine)) terminate (IDS_TERM_INITALIZE,get_string(IDS_TERM_STARUPFILE));
//
//  Main message loop.
//
  while (GetMessage(&msg,NULL,0,0)) {
    if (!TranslateAccelerator(main_window,haccel,&msg)) {
      TranslateMessage (&msg);
      DispatchMessage  (&msg);
    }
  }
#ifdef WINCE_PPC                    // Cleanup for PPC task blocking.
  if (mutex) CloseHandle (mutex);
#endif WINCE_PPC
  CoUninitialize ();                // Cleanup COM interface.
  return (msg.wParam);
}
















