//-------------------------------------------------------------------//
//                                                                   //
//  JFC Copyright (C) Glenn Rosenthal, 1999,2000,2001.               //
//  All rights reserved.                                             //
//                                                                   //
//-------------------------------------------------------------------//

//-------------------------------------------------------------------
//
//  This module implements the JFC_window class.  This class fucntions to manage
//  the main window, and control all of the Japanese text box controls, as well 
//  as the main button placement.
//

#include "jfc_main.h"
#include "jfc_conf.h"
#include "jfc_stat.h"
#include "jfc_wind.h"
#include "jfc_misc.h"

extern HWND command_bar;    // From the main procedure, this is the window pointer to the command bar
static HWND right_window,wrong_window,show_window;

//-------------------------------------------------------------------
//
//  Exported data.
//

class JFC_window jfc_window;

//-------------------------------------------------------------------
//
//  Begin class JFC_window
//

//
//  Constructor
//
JFC_window::JFC_window () {
  number_windows  = 0;
  visible_windows = 0;
  return;
}

//
//  This is actually the main working routine.  This set the number of visisble windows,
//  and positions them in the display screen.
//
//      count -- Number of visible windows.  A value of -1 can be used to keep the number
//               of windows constaint.
//
void JFC_window::adjust (int count) {
  RECT rect;
  int i,top;
  GetClientRect (main_window,&rect);                        // Get width of main window.
  if (count == -1) count = number_windows; else number_windows = count;
//
//  Position the button windows.
//
  top = top_space;
#ifdef WINCE
  SendMessage (command_bar,TB_SETSTATE,IDM_CARD_SHOW ,MAKELONG(count ? TBSTATE_ENABLED : 0,0));
  SendMessage (command_bar,TB_SETSTATE,IDM_CARD_RIGHT,MAKELONG(count ? TBSTATE_ENABLED : 0,0));
  SendMessage (command_bar,TB_SETSTATE,IDM_CARD_WRONG,MAKELONG(count ? TBSTATE_ENABLED : 0,0));
#else WINCE
  MoveWindow   (show_window ,0,0,rect.right/3,top,true);
  MoveWindow   (right_window,rect.right/3+1,0,rect.right/3,top,true);
  MoveWindow   (wrong_window,2*(rect.right/3)+2,0,rect.right-2*(rect.right/3)-2,top,true);
  EnableWindow (show_window ,count);
  EnableWindow (right_window,count);
  EnableWindow (wrong_window,count);
#endif WINCE
  EnableMenuItem (hmenu,IDM_CARD_SHOW ,MF_BYCOMMAND | (count ? MF_ENABLED : MF_GRAYED));
  EnableMenuItem (hmenu,IDM_CARD_RIGHT,MF_BYCOMMAND | (count ? MF_ENABLED : MF_GRAYED));
  EnableMenuItem (hmenu,IDM_CARD_WRONG,MF_BYCOMMAND | (count ? MF_ENABLED : MF_GRAYED));
//
//  Position the main windows.
//
  top += kana_font.vspace;
  for (i = 0; (i < NUMBER_WINDOWS) && (i < count); i++) {               // Posiiton each window 
    windows[i].move (kana_font.hspace,top,rect.right-2*kana_font.hspace);
    windows[i].show ();                                                 // Show the windows.
#ifdef WINCE
    int tt;
    tt = top;
    top += windows[i].height+kana_font.vspace;
    if (tt+kana_font.vspace+kana_font.lheight > rect.bottom-jfc_stat.height) break;   
int this_font_spacing_needs_to_be_worked_out_correctly;
#else WINCE
    top += windows[i].height+kana_font.vspace;
    if (top > rect.bottom-jfc_stat.height) break;   
#endif WINCE
  }
  visible_windows = i;                                      // Indicate visible windows
  for (; i < NUMBER_WINDOWS; i++) windows[i].hide ();       // Hide all other windows.
}

//
//  This routine initializes the class.  This consists of opening a number of windows
//  and getting them ready for use.
//
//      RETURN -- A non-zero value indicates an error.
//
int JFC_window::initialize () {
  int i;
//
//  Generate the button windows for non-CE machines.  For CE machines the buttons are
//  in the command bar.
//
#ifndef WINCE
  show_window  = CreateWindow (TEXT("BUTTON"),NULL,WS_CHILD | WS_VISIBLE,0,0,10,10,main_window,(HMENU) IDM_CARD_SHOW ,instance,NULL);
  right_window = CreateWindow (TEXT("BUTTON"),NULL,WS_CHILD | WS_VISIBLE,0,0,10,10,main_window,(HMENU) IDM_CARD_RIGHT,instance,NULL);
  wrong_window = CreateWindow (TEXT("BUTTON"),NULL,WS_CHILD | WS_VISIBLE,0,0,10,10,main_window,(HMENU) IDM_CARD_WRONG,instance,NULL);
  if (!right_window || !wrong_window || !show_window) return (true);
  SetWindowText (right_window,get_string(IDS_RIGHT));
  SetWindowText (wrong_window,get_string(IDS_WRONG));
  SetWindowText (show_window ,get_string(IDS_SHOW ));
#endif WINCE
//
//  Figure out how much space is requried at the top of the display.  We save this because
//  it gets used a lot.
//
#ifdef WINCE
  top_space = CommandBar_Height(command_bar);
#else  WINCE
  top_space = (3*sysfont_height)/2;
#endif WINCE
//
//  Open text windows and initialize display.
//
  for (i = 0; i < NUMBER_WINDOWS; i++) {        // Open windows.
    if (windows[i].open(i+1)) return (true);
  }
  adjust (0);                                   // Setup display.
  return (false);  
}

//
//  Small routine to set active window.
//
//      id     -- ID to change to.  This is auto-wrapped.
//      expand -- If non-zero this allows the entry to be auto-expanded.
//
void JFC_window::set (int id,int expand) {
  if (id == 0              ) id = visible_windows;
  if (id >  visible_windows) id = 1;
  SetFocus (windows[id-1].window);
//
//  Auto expand option.
//
  if (jfc_config.cfg.auto_expand && expand && (MODE_SWAP == windows[id-1].get_mode()) && (windows[id-1].multi == 0)) {
    windows[id-1].multi = 1;
    windows[id-1].view (true);
  }
  return;
}

//
//  This routine adjusts the spacing of the windows from a window downward.  This is as a
//  responce to changes in the displayed information in the window.
//
//      start -- Window number to start at.
//
void JFC_window::space (int start) {
  int i,top = top_space;
//
//  Position the main windows.
//
  top += kana_font.vspace;
  for (i = 0; (i < NUMBER_WINDOWS) && (i < number_windows); i++) {      // Posiiton each window 
    windows[i].adjust (kana_font.hspace,top);
    top += windows[i].height+kana_font.vspace;
  }
  return;
}





